
/* advext demo by Kevin Venzke, Dec 11 2005 */
/* demonstrates some features of advext.t */

#include <adv.t>
#include <advext.t>
#include <std.t>

replace init: function
{
    commonInit();
    
    "\b\bAdvext demo by Kevin Venzke\b\b";

    //This is necessary, since afterTurn() is called from here:
    setdaemon(turncount, nil);

    parserGetMe().location:=startroom;
    startroom.lookAround(true);
    startroom.isseen:=true;
    scoreStatus(0, 0);
}

startroom: room
    //I'll define both rdesc and sdesc, but just the following is fine:
    //sdesc = "Equipment Room"
    rdesc = "Equipment Room"
    sdesc = "equipment room"
    
    //"equipment room" should take "an" rather than "a"
    isAn = true
    
    //Vocabulary is optional:
    adjective = 'equipment'
    noun = 'room'
    
    //The throne room is adjacent to this.
    adjrooms = [throneroom]
    
    ldesc = "Equipment abounds in this room. Choose your props wisely, mighty hero.
        \n\tThere are gray bricks in the floor, totally unremarkable things. Some candy
        wrappers litter them and blow about in the wind. 
        \n\tThe throne room is north. You may humbly approach using GO NORTH, or GO THRONE
        ROOM. It is your prerogative. "
    
    north = throneroom
    
    exits = 'north'
;

//The bricks and wrappers needn't be referred to. They are just descriptive color.
bricksandodor: unnec
    location = startroom
    //They don't need any sdesc.
    adjective = 'gray' 'candy' noun = 'bricks' 'wrapper' 'wrappers' 'wind'
;

shieldpolisher: fixeditem
    location = startroom
    sdesc = "shield polisher"
    adjective = 'shield'
    noun = 'polisher'
    
    //I can give this an origdesc even though it is not an origitem. That creates
    //no problems because this can't be moved.
    origdesc = "\n\tA shield polisher is fastened to the center of one wall. "
    
    ldesc = "The shield polisher has a plastic bit that responds only to being examined or
        eaten. If you attempt any other interaction, the game shall pretend you were 
        referring to the polisher as a whole. "
;

plasticbit: component
    //This is all we need to do to make it a component of the polisher:
    location = shieldpolisher
    
    sdesc = "plastic bit"
    adjective = 'plastic'
    noun = 'bit'
    
    //The player can examine this:
    verDoInspect(a) = {}
    ldesc = "The plastic bit is of unknown significance... But it responds to being eaten. "
    
    //The player can try to eat it:
    verDoEat(a) = "How's that?\ You can't eat plastic, even if permitted! "
;

//The sword is an origitem because it has a special initial description.
sword: origitem
    location = startroom
    sdesc = "Dangerous Sword of Jumping"
    adjective = 'dangerous'
    noun ='sword' 'jumping'

    origdesc = "\n\tThe Dangerous Sword of Jumping awaits you climactically, hero! In 
        the middle of the floor! "

    //For fun, let's say that the player must hold the sword (or attempt to take it implicitly)
    //in order to examine it:
    taketolook = true
    
    //Advext automatically remembers whether we've examined this, so we can vary the
    //response based on this:
    ldesc = {
        if(not self.havelooked)
            "To your surprise and horror, \v";
        "It's a very unremarkable sword. However, you must not jump while holding it. ";
    }
    
    //Let's customize the text for taking/dropping this:
    taken = "You lift the sword with a grunt and wield it mightily, flailing like a helicopter! "
    dropped = "The sword drops with a clang. You exhale in relief. "

    //If the player tries to jump while holding the sword, well, that's dangerous.
    scopeAction(a,v,d,p,i) = {
        //If the player is jumping and the sword is in the player:
        if(a=parserGetMe() and location=a and v=jumpVerb) {
            "Stop! It's very dangerous to jump with a sword! ";
            exit;
        }
    }
    
    //I want a message to display whenever you walk around with the sword. So
    //we need to have this registered in dt.msgs. We could do that in the init
    //function, or we could do it in an auto property. The auto property makes
    //it pretty easy; the downside is that it will always be run, every turn.
    
    //The only problem is that this auto won't run prior to the first turn.
    //So if the player travels on the first turn, this won't run. But that's okay,
    //because the player wouldn't have the sword then.
    
    auto = {
        if( find(dt.msgs,self)=nil ) {
            dt.msgs:=dt.msgs + self;
//Unremark the below if you want to see when this happens:
//            "\n[Sword added to dt.msgs.]\n";
        }
    }
    
    duringTravel(dest) = {
        //Show a message if player has the sword and dest isn't a nestedroom.
        if(location=parserGetMe() and not isclass(dest,nestedroom)) {
            "You drag the sword behind you, making a horrible scraping sound.\b";
        }
    }
;

throneroom: room
    rdesc = "Throne Room"
    sdesc = "throne room"
    adjective = 'throne'
    noun = 'room'
    
    adjrooms = [startroom kitchennook]
    
    //We don't have to define ver/do methods for Enter, if we're alright with a
    //simple travelTo(thisroom).
    
    ldesc = "This is where the mighty Queen Felicia interrogates the merchants! Tremble! You can go
        south back to the equipment room. There's a kitchen nook on one wall which you can GO to, and
        still be able to observe the Queen. "
        
    south = startroom
    
    ex = "You can go south, or to the kitchen nook. "
    
    //Let's have a dramatic message when you take something from here:
    takenfrom(obj) = {
        "You remove <<obj.thedesc>> from the throne room! ";
        return true;
    }
;

felicia: Actor
    sdesc = "Queen Felicia"
    isProper = true   //We don't want a/the Felicia.
    location = throneroom
    adjective = 'queen'
    noun = 'felicia'
    
    ldesc = "Queen Felicia has grape-colored jewels and an ivory comb in her hair. I guess
        you could ask for her stuff. "
    
    //Felicia is visible from the kitchen or boat, where she isn't reachable. We can
    //customize fardesc for this.
    fardesc = "From <<parserGetMe().location.thedesc>>, one must squint to make out even 
        vague details of Queen Felicia. (Her jewels and comb are just as visible as they were 
        before, though.) "

    //We want a message out of her the first time she's visible to us.
    haschattered = nil
    chatter = {
        if(not haschattered) {
            "\b\"Hero! The king ails and I die myself within forty years. Bring to me the fountain
            of youth! I bestow upon you a boat if you ask for it!\"";
            setit(self,2);
            haschattered:=true;
            return;
        }
    }
    
    //You can ask Felicia for things:
    askForWord(w,l) = {
        if(w='jewel' or w='jewels') {
            "\"Your jewels are so pretty,\" you inform Queen Felicia.
            \b\"You'd best forget about them!\"\ she replies. ";
            return true;
        }
        if(w='comb') {
            "\"Hmm, I could use a comb such as yours,\" you hint.
            \b\"Find your own,\" Felicia replies. ";
            return true;
        }
        if((w='ship' or w='boat') and ship.location=nil) {
            "Felicia snaps her fingers. \"I summon a ship for you!\"
            \bYou hear a thud to the south. ";
            ship.moveInto(startroom);
            //We'll have special text for when the ship is noticed:
            startroom.haschanged:=noticeship;
            return true;
        }
        return nil;
    }
    
    //Let's have Felicia respond randomly here:
    disavowAskFor = "<<feliciacantgive.getmsg>>"
    
    //Felicia doesn't want you taking off with the evidence of her husband's infidelity.
    //And trumpets blare whenever you do something with her.
    scopeAction(a,v,d,p,i) = {
        if(a=parserGetMe() and isclass(v,travelVerb) and evidence.location=a) {
            "\"Halt! Leave that evidence right here!\"\ orders the Queen! ";
            exit;
        }
        if((a=self or d=self or i=self) and v<>inspectVerb) {
            "Trumpets blare as you prepare to do something with Queen Felicia!\b";
            return;
        }
    }
;

feliciacantgive: shuffler
    msglist = [
        '"How can I give you that?"\ asks Felicia. '
        '"You can\'t have *that*!"\ groans Felicia. '
        'Felicia denies your odd request. '
        '"I\'ve never even heard of that," Felicia confesses. '
    ]
;

//Jewels and comb are genitems. Most interactions fail for a common reason.
jewels: genitem
    location = felicia
    sdesc = "grape-colored jewels"
    isThem = true
    //I don't need to define a special adesc, since isThem is set.
    adjective = 'grape' 'colored' 'grape-colored'
    noun = 'jewel' 'jewels'

    verDoInspect(a) = {}
    gendesc = "You can't do much with the jewels while Felicia has them. "
;
comb: genitem
    location = felicia
    sdesc = "ivory comb"
    isAn = true
    adjective = 'ivory'
    noun = 'comb' 'hair'
    
    verDoInspect(a) = {}
    gendesc = "You can't do much with the comb while Felicia has it. "
;

//The ship can be pushed around.
ship: pushable, chairitem
    isdroploc = true
    sdesc = "boat"
    adjective = 'little' noun = 'ship' 'boat'
    
    isListed = true
    
    ldesc = "It's a little boat. If you get in it, you can leave by typing GET OUT,
        GET OFF, STAND, UP, DOWN, or OUT. You can also PUSH BOAT NORTH, etc. "
    
    doSynonym('Board') = 'Enter'
    
    pushmsg(dest) = "It's hard work, but you push the boat in front of you.\b"
;

//When startroom's haschanged is set to this object, these messages appear once.
noticeship: changenotice
    beforelook = "What's this? Something new has appeared here!\b"
    duringlook = "Why, it looks like there's a boat here!\n\t"
    afterlook = "\bYou marvel at the little boat with awe. "
;

kitchennook: room
    adjrooms = [throneroom]
    rdesc = "Kitchen Nook"
    sdesc = "kitchen nook"
    
    adjective = 'kitchen'
    noun = 'nook'
    
    out = throneroom
    exits = 'out, back to the throne room'
    
    //You can see Felicia and her stuff from here.
    cansee = [felicia]
    
    ldesc = "This is where Queen Felicia may strangle a chicken if she deems it
        appropriate. She observes you watchfully from where she sits.
        \n\tA note says:\ \"Take my orange and you'll hang!\" (Due to this note, I'll
        not assume you wish to take the orange if you just type TAKE or TAKE ALL.)
        \n\t(Also, there's a picture of an orange on the wall. If you refer to ORANGE I'll
        assume you don't mean the picture, unless the verb is BREAK in which case
        I'll assume it must be the picture that you mean.)"
;

orangepicture: genitem
    location = kitchennook
    sdesc = "picture of an orange"
    noun = 'picture' 'orange' 'wall'

    disambig(verb,asDobj,wordlist,objlist) = {
        //If it's ambiguous and the verb is break, it should be this object.
        if(verb=breakVerb) return 120;
        
        //Otherwise it probably shouldn't be this object.
        return 0;
    }
    
    verDoBreak(a) = {}
    doBreak(a) = "I guess you mean the picture, just as I said I would. "
    
    //Inspecting is ok:
    verDoInspect(a) = {}
    ldesc = "I see you're looking at the picture. You won't have much luck trying other
        things with it, though. "

    gendesc = "The picture of an orange is beyond your reach. (Or it's only visible
        in the distance. Or it's not yours to meddle with. Or it's just not implemented
        very well.) "
;

orange: item
    location = kitchennook
    sdesc = "orange"
    noun = 'orange'
    isAn = true
    
    //The game mustn't take the orange by default, since this is a dangerous act.
    notDefault(verb,asDobj) = {
        if(verb=takeVerb and asDobj) return true;
        return nil;
    }
    
    //Customize taken message:
    taken = "Taken. (I guess you won't be hung after all.) "
    
    //The orange chatters that you should take it
    chatter = {
        if(_rand(4)=1 and self.location<>parserGetMe()) 
            "\b\"Take... me...\"\ whispers the orange. ";
    }
    
    //If you want to eat the orange, you'll have to take it first.
    verDoEat(a) = {}
    doEat(a) = {
        if(not self.autotake(a)) return;
        "You eat the orange! Bam, it's gone! ";
        self.moveInto(nil);
    }
;

tangerine: item
    location = kitchennook
    sdesc = "tangerine"
    noun = 'tangerine'
;

evidence: item
    location = throneroom
    sdesc = "evidence of the King's infidelity"
    adesc = (self.sdesc)
    adjective = 'kings' 'king\'s'
    noun = 'evidence' 'infidelity'
;